# NACA 0012-34 airfoil simulation

## Geometry generation

The script *generate_airfoil.py* creates two files:
- **naca0012-34.stl**: geometry serving a input STL for *snappyHexMesh*
- **NACA-0012-34.pdf**: a simple plot comparing the profile data generated by the script against external profile data stored in *naca0012-34.csv*

## Simulation conditions

- Reynolds number $Re=3.0\times 10^6$
- angle of attack $\alpha = 3.5^\circ$
- Mach number $Ma = 0.73$

The Mach number is defined as the ratio between free-stream velocity $U_{in}$ and the speed of sound $c$:

$$ Ma = U_{in}/c \text{ with } c=\sqrt{\kappa R_s T_{in}} $$

Some useful expression are:
- specific gas  constant: $R_s = R/M = 8.3145 J/(mol K) / 28.9 g/mol  \cdot 1000 g/kg = 287.7 J/(kg K)$
- head capacity ratio of dry air at room temperature: $\kappa = 1.4$
- speed of sound: $c=\sqrt{1.4 \cdot 287.7 \cdot 293} = 343.5$
- transonic inlet velocity: $U_{in} = Ma \cdot c = 0.73\cdot 343.5 = 250.8$
- free-stream density: $\rho = p_{inf}/(R_s T_{in}) = 10^5 / (287.7 \cdot 293) = 1.186 kg/m^3$
- kinematic viscosity based on free-stream values: $\nu = \mu / \rho = 1.82\times 10^{-5} / 1.186 = 1.5342\times 10^{-5} m^2/s$
- characteristic length (chord length of airfoil): $L = Re \nu / U_{in} = 3\times 10^6 \cdot 1.5342\times 10^{-5} / 250.8 = 0.18352$